set -e

#!/bin/bash
if [ $EUID != 0 ]; then
    sudo "$0" "$@"
    exit $?
fi
# must correct configure all paths
LOGO_IMAGE_PACKER=./bin/logo_img_packer
AML_IMAGE_PACKER=./bin/aml_image_v2_packer
SIMG2IMG=./bin/simg2img
IMG2SIMG=./bin/img2simg
SIGNAPK=./bin/signapk.jar
DUMPKEY=./bin/dumpkey.jar
MKBOOTFS=./bin/mkbootfs
MKBOOTIMG=./bin/mkbootimg
UNPACKBOOTIMG=./bin/unpackbootimg
IMG2SDAT=./bin/img2sdat.py
UPDATER_BINARY=./etc/update-binary
UPDATER_SCRIPT=./etc/updater-script

# init default values
TEMP_FOLDER=./temp
OTA="./fullOTA.zip"

# chmod for all needed files
chmod 755 ./bin/*
chmod 644 ./etc/*
chmod 644 ./lib/*
chmod 644 ./lib64/*

HELP="Amlogic customization utility. Version 1.0.9
Usage: $0 -i input_file -o output_file -rk key_files
Where:
    -i|--input           Location of the input file (must be specified)
    -o|--output          Location of the output file (must be specified)
    -b|--bootanimation   New bootanimation file
    -d|--delete          Apps that will be deleted (names separated by comma)
    -p|--preinstall      Apps that will be added (paths separated by comma)
    -rk|--recoverykey    Recovery keys to resign OTA package (name without format; must be specified)
                         Private and plublic key must be in the same folder and must have
                         the same name (example: -rk ./keys/releasekey)
    -pk|--platformkey    Platform keys to resign system image (name without format)
                         Private and plublic key must be in the same folder and must have
                         the same name (example: -pk ./keys/platformkey)
    -a|--autostart       Name of the app activity that will be started at boot
    -u|--updateurl       Update URL for OTA
    -n|--name            Name of the device (ro.product.name)
    -t|--temp            Temp folder location (default: ./temp)
    -z|--zipname         Output name of the OTA zip file (default: ./fullOTA.zip)
    -dn|--devicename     Device name that will be checked by the updater-script
                         (default: ro.build.device of the input firmware)
    -bl|--bootlogo       Boot logo to replace default image on start
                         Image must be in BMP format (16bit, RGB565, preferably 320х320)
                         and not bigger than 512KB


Examples:
     $0 -i ./aml_upgrade_package.img -o temp.img -t ./test -d Skype,AppInstaller -b ./bootanimation.zip -p ./unpackbootimg -n nexus -rk ./releasekey
     $0 -i ./aml_upgrade_package.img -o temp.img -t ./test --recoverykey ./releasekey"

# check utilities
if [ ! -f $LOGO_IMAGE_PACKER ]; then
    echo "Amlogic logo image packer not found: $LOGO_IMAGE_PACKER"
    exit 1
fi

if [ ! -f $AML_IMAGE_PACKER ]; then
    echo "Amlogic image packer not found: $AML_IMAGE_PACKER"
    exit 1
fi

if [ ! -f $SIMG2IMG ]; then
    echo "simg2img not found: $SIMG2IMG"
    exit 1
fi

if [ ! -f $IMG2SIMG ]; then
    echo "img2simg not found: $IMG2SIMG"
    exit 1
fi

if [ ! -f $SIGNAPK ]; then
    echo "signapk.jar not found: $SIGNAPK"
    exit 1
fi

if [ ! -f $DUMPKEY ]; then
    echo "dumpkey.jar not found: $DUMPKEY"
    exit 1
fi

if [ ! -f $MKBOOTFS ]; then
    echo "mkbootfs not found: $MKBOOTFS"
    exit 1
fi

if [ ! -f $MKBOOTIMG ]; then
    echo "mkbootimg not found: $MKBOOTIMG"
    exit 1
fi

if [ ! -f $UNPACKBOOTIMG ]; then
    echo "unpackbootimg not found: $UNPACKBOOTIMG"
    exit 1
fi

if [ ! -f $IMG2SDAT ]; then
    echo "img2sdat not found: $IMG2SDAT"
    exit 1
fi

if [ ! -f $UPDATER_BINARY ]; then
    echo "updater-binary not found: $UPDATER_BINARY"
    exit 1
fi

if [ ! -f $UPDATER_SCRIPT ]; then
    echo "updater-script not found: $UPDATER_SCRIPT"
    exit 1
fi

#get attributes
while test $# -gt 0; do
    case $1 in
	-h|--help)
	    echo "$HELP"
	    exit 0
	    ;;
	-i|--input)
	    shift
	    IN_IMAGE=$1
	    if [ ! -f $IN_IMAGE ]; then
	    echo "File not found: $IN_IMAGE"
	    exit 1
	    fi
	    shift
	    ;;
	-o|--output)
	    shift
	    OUT_IMAGE=$1
	    if [ -f $OUT_IMAGE ]; then
	    echo "File already exists: $OUT_IMAGE. Try another output name"
	    exit 1
	    fi
	    shift
	    ;;
	-b|--bootanimation)
	    shift
	    BOOTANIMATION=$1
	    if [ ! -f $BOOTANIMATION ]; then
	    echo "File not found: $BOOTANIMATION"
	    exit 1
	    fi
	    shift
	    ;;
	-d|--delete)
	    shift
	    DELETE_APPS=$(echo $1 | sed 's/,/\n/g')
	    if [ -z "$DELETE_APPS" ]; then
	    echo "Please, specify apps to delete!"
	    exit 1
	    fi
	    shift
	    ;;
	-p|--preinstall)
	    shift
	    PREINSTALL_APPS=$(echo $1 | sed 's/,/\n/g')
	    if [ -z "$PREINSTALL_APPS" ]; then
	    echo "Please, specify apps to preinstall!"
	    exit 1
	    fi
	    shift
	    ;;
	-rk|--recoverykey)
	    shift
	    RECOVERY_KEY=$1
	    if [ ! -f ${RECOVERY_KEY}.x509.pem ] || [ ! -f ${RECOVERY_KEY}.pk8 ]; then
	    echo "Some recovery key files not found!"
	    exit 1
	    fi
	    shift
	    ;;
	-pk|--platformkey)
	    shift
	    PLATFORM_KEY=$1
	    if [ ! -f ${PLATFORM_KEY}.x509.pem ] || [ ! -f ${PLATFORM_KEY}.pk8 ]; then
	    echo "Some platform key files not found!"
	    exit 1
	    fi
	    shift
	    ;;
	-a|--autostart)
	    shift
	    AUTOSTART=$1
	    if [ -z $AUTOSTART ]; then
	    echo "Autostart is empty!"
	    exit 1
	    fi
	    shift
	    ;;
	-u|--updateurl)
	    shift
	    UPDATEURL=$1
	    if [ -z $UPDATEURL ]; then
	    echo "URL is empty!"
	    exit 1
	    fi
	    shift
	    ;;
	-n|--name)
	    shift
	    NAME=$1
	    if [ -z $NAME ]; then
	    echo "Name is empty!"
	    exit 1
	    fi
	    shift
	    ;;
	-t|--temp)
	    shift
	    TEMP_FOLDER=$1
	    if [ -d $TEMP_FOLDER ]; then
	    echo "Temp folder is exists! Try another temp folder."
	    exit 1
	    fi
	    shift
	    ;;
	-z|--zipname)
	    shift
	    OTA=$1
	    if [ -z $OTA ]; then
	    echo "OTA zip file name is empty!"
	    exit 1
	    fi
	    shift
	    ;;
	-dn|--devicename)
	    shift
	    DEVICE=$1
	    if [ -z $DEVICE ]; then
	    echo "Device name can not be empty."
	    exit 1
	    fi
	    shift
	    ;;
    -bl|--bootlogo)
        shift
        BOOTLOGO=$1
        if [ ! -f $BOOTLOGO ]; then
        echo "File not found: $BOOTLOGO"
        exit 1
        fi
        BOOTLOGO_SIZE=$(stat -c%s $(pwd)/$BOOTLOGO)
        ((${BOOTLOGO_SIZE} > 524288)) && \
        echo "File \"$BOOTLOGO\" is too big. Please make another boot logo file that not bigger than 512KB" && \
        exit 1
        shift
        ;;
	*)
	    echo "Unknown attribute: $1"
	    exit 1
	    ;;
    esac
done

if [ -z $IN_IMAGE ] || [ -z $OUT_IMAGE ] || [ -z $RECOVERY_KEY ]; then
echo "$HELP"
exit 1
fi

if [ -d $TEMP_FOLDER ]; then
echo "Temp folder is exists! Try another temp folder. (path: $TEMP_FOLDER)"
exit 1
fi

if [ -f $OTA ]; then
echo "File $OTA already exists. Try another OTA zip name."
exit 1
fi

echo "Input image: $IN_IMAGE"
echo "Output image: $OUT_IMAGE"

if [ ! -z $BOOTLOGO ]; then
echo "New boot logo: $BOOTLOGO"
fi

if [ ! -z $BOOTANIMATION ]; then
echo "New bootanimation: $BOOTANIMATION"
fi

if [ ! -z $UPDATEURL ]; then
echo "New update URL: $UPDATEURL"
fi

if [ ! -z $AUTOSTART ]; then
echo "New austotart: $AUTOSTART"
fi

echo "Recovery key: $RECOVERY_KEY"

if [ ! -z $PLATFORM_KEY ]; then
echo "Platform key: $PLATFORM_KEY"
fi

if [ ! -z $NAME ]; then
echo "New device name: $NAME"
fi

for app in $PREINSTALL_APPS; do
echo "Preinstall app: $app"
done

for app in $DELETE_APPS; do
echo "Delete app: $app"
done

echo "OTA zip: $OTA"

if [ ! -z $DEVICE ]; then
echo "OTA device name: $DEVICE"
fi

mkdir $TEMP_FOLDER

echo "Unpacking image..."
$AML_IMAGE_PACKER -d $IN_IMAGE $TEMP_FOLDER

#repack logo image if needed
if [ ! -z $BOOTLOGO ]; then
    echo "Unpacking logo..."
    mkdir $TEMP_FOLDER/logo
    $LOGO_IMAGE_PACKER -d $TEMP_FOLDER/logo.PARTITION $TEMP_FOLDER/logo
    rm -rf $TEMP_FOLDER/logo.PARTITION
    cp -f $BOOTLOGO $TEMP_FOLDER/logo/bootup
    echo "Repacking logo.PARTITION..."
    $LOGO_IMAGE_PACKER -r $TEMP_FOLDER/logo $TEMP_FOLDER/logo.PARTITION
    rm -rf $TEMP_FOLDER/logo
fi

#if [ ! -z $RECOVERY_KEY ]; then
echo "Unpacking recovery..."
mkdir $TEMP_FOLDER/recovery
$UNPACKBOOTIMG -i $TEMP_FOLDER/recovery.PARTITION -o $TEMP_FOLDER/recovery
rm -rf $TEMP_FOLDER/recovery.PARTITION

echo "Decompressing recovery ramdisk..."
mkdir $TEMP_FOLDER/recovery/ramdisk
CUR=$(pwd)
cd $TEMP_FOLDER/recovery/ramdisk
gunzip -c ../recovery.PARTITION-ramdisk.gz | cpio -i
cd $CUR
rm -rf $TEMP_FOLDER/recovery/recovery.PARTITION-ramdisk.gz

echo "Replacing recovery keys..."
java -jar $DUMPKEY ${RECOVERY_KEY}.x509.pem > $TEMP_FOLDER/keys
rm -rf $TEMP_FOLDER/recovery/ramdisk/res/keys
mv  $TEMP_FOLDER/keys $TEMP_FOLDER/recovery/ramdisk/res/keys

echo "Compressing recovery ramdisk..."
$MKBOOTFS $TEMP_FOLDER/recovery/ramdisk | gzip > $TEMP_FOLDER/recovery/recovery.PARTITION-ramdisk.gz
rm -rf $TEMP_FOLDER/recovery/ramdisk

echo "Repacking recovery.PARTITION..."
RECOVERY_FOLDER=$TEMP_FOLDER/recovery
KERNEL=${RECOVERY_FOLDER}/recovery.PARTITION-zImage
RAMDISK=${RECOVERY_FOLDER}/recovery.PARTITION-ramdisk.gz
SECOND=${RECOVERY_FOLDER}/recovery.PARTITION-second
CMDLINE=
BOARD=
BASE=
PAGESIZE=

CMDLINE=$(cat ${RECOVERY_FOLDER}/recovery.PARTITION-cmdline)
BOARD=
BASE=$(cat ${RECOVERY_FOLDER}/recovery.PARTITION-base)
PAGESIZE=$(cat ${RECOVERY_FOLDER}/recovery.PARTITION-pagesize)

FINAL_ATTRS="--kernel $KERNEL --ramdisk $RAMDISK --second $SECOND"

if [ ! -z $CMDLINE ]; then
FINAL_ATTRS="${FINAL_ATTRS} --cmdline $CMDLINE"
fi

if [ ! -z $BASE ]; then
FINAL_ATTRS="${FINAL_ATTRS} --base $BASE"
fi

if [ ! -z $PAGESIZE ]; then
FINAL_ATTRS="${FINAL_ATTRS} --pagesize $PAGESIZE"
fi

$MKBOOTIMG \
    $FINAL_ATTRS \
    -o $TEMP_FOLDER/recovery.PARTITION

rm -rf  $TEMP_FOLDER/recovery

#fi #end if [ ! -z $RECOVERY_KEY ]

echo "Decompressing system.PARTITION..."
$SIMG2IMG $TEMP_FOLDER/system.PARTITION $TEMP_FOLDER/system_raw.img
rm -rf $TEMP_FOLDER/system.PARTITION

echo "Mount system_raw_img..."
mkdir $TEMP_FOLDER/system && mount -t ext4 $TEMP_FOLDER/system_raw.img $TEMP_FOLDER/system

#delete unneeded apps
for app in $DELETE_APPS; do
APP=$(find $TEMP_FOLDER/system/app -type d -name $app)
PRIV_APP=$(find $TEMP_FOLDER/system/priv-app -type d -name $app)
PREINSTALL=$(find $TEMP_FOLDER/system/preinstall -type f -name *${app}*)
for name in $APP; do
echo "Delete app $name"
rm -rf $name
done
for name in $PRIV_APP; do
echo "Delete app $name"
rm -rf $name
done
for name in $PREINSTALL; do
echo "Delete app $name"
rm -rf $name
done
done

#add preinstall apps
mkdir -p $TEMP_FOLDER/system/preinstall
if [ ! -d "$TEMP_FOLDER/system/preinstall" ]; then
  echo "Can not create folder for preinstall apps!"
  exit 0
fi
for app in  $PREINSTALL_APPS; do
echo "Copy $app to preinstall..."
cp -f $app $TEMP_FOLDER/system/preinstall/
done

#replace bootanimation
if [ ! -z $BOOTANIMATION ]; then
echo "Replacing bootanimation.zip..."
rm -rf $TEMP_FOLDER/system/media/bootanimation.zip
cp -f $BOOTANIMATION $TEMP_FOLDER/system/media/bootanimation.zip
fi

#resign system image
if [ ! -z $PLATFORM_KEY ]; then
echo "Moving files to local drive..."
mv $TEMP_FOLDER/system/app $TEMP_FOLDER/app
mv $TEMP_FOLDER/system/priv-app $TEMP_FOLDER/priv-app
mv $TEMP_FOLDER/system/framework $TEMP_FOLDER/framework

FRAMEWORK_FOLDER=$TEMP_FOLDER/framework
unzip -d $FRAMEWORK_FOLDER/extracted $FRAMEWORK_FOLDER/framework-res.apk META-INF/*.RSA
RSA_FILE=$(find ${FRAMEWORK_FOLDER}/extracted/META-INF -type f -name '*.RSA')
FRAMEWORK_RES_SERIAL=$(keytool -printcert -v -file $RSA_FILE | grep SerialNumber | cut -c 19-23)
echo "Framework-res serial detected: $FRAMEWORK_RES_SERIAL"
rm -rf $FRAMEWORK_FOLDER/extracted

ALL_APKS=$(find $TEMP_FOLDER/app $TEMP_FOLDER/priv-app -type f -name '*.apk')
ALL_FRAMEWORK_FILES=$(find $TEMP_FOLDER/framework -type f -name '*.apk' -or -name '*.jar')

echo "Resigning all system apps..."
for name in $ALL_APKS; do
unzip -d ${name}.extracted $name META-INF/*.RSA
RSA_FILE=$(find ${name}.extracted/META-INF -type f -name '*.RSA')
APK_SERIAL=$(keytool -printcert -v -file ${RSA_FILE} | grep SerialNumber | cut -c 19-23)
echo "APK Serial: $APK_SERIAL"
rm -rf ${name}.extracted
if [[ "$FRAMEWORK_RES_SERIAL" == "$APK_SERIAL" ]]; then
echo "Resigning $name"
mv $name ${name}_old
java -jar $SIGNAPK ${PLATFORM_KEY}.x509.pem ${PLATFORM_KEY}.pk8 ${name}_old $name
rm ${name}_old
else
echo "Skip $name"
fi
done

echo "Resigning all framework files..."
for name in $ALL_FRAMEWORK_FILES; do
echo "Resigning $name"
mv $name ${name}_old
sudo java -jar $SIGNAPK ${PLATFORM_KEY}.x509.pem ${PLATFORM_KEY}.pk8 ${name}_old $name
rm ${name}_old
done

echo "Replacing files in system_raw.img..."
mv $TEMP_FOLDER/app $TEMP_FOLDER/system/app
mv $TEMP_FOLDER/priv-app $TEMP_FOLDER/system/priv-app
mv $TEMP_FOLDER/framework $TEMP_FOLDER/system/framework

fi #end if [ ! -z $PLATFORM_KEY ]

echo "Create and replace otacerts.zip..."
zip $TEMP_FOLDER/otacerts.zip ${RECOVERY_KEY}.x509.pem
rm -rf $TEMP_FOLDER/system/etc/security/otacerts.zip
mv $TEMP_FOLDER/otacerts.zip $TEMP_FOLDER/system/etc/security/otacerts.zip

#replace update url
if [ ! -z $UPDATEURL ]; then
echo "Changing update url..."
CURRENTUPDATEURL=$(grep ^ro.build.otaurl $TEMP_FOLDER/system/build.prop)
if [ ! -z $CURRENTUPDATEURL ]; then
sed '/ro\.build\.otaurl.*/d'  $TEMP_FOLDER/system/build.prop >  $TEMP_FOLDER/system/build.prop.new
rm -rf $TEMP_FOLDER/system/build.prop
mv $TEMP_FOLDER/system/build.prop.new $TEMP_FOLDER/system/build.prop
fi
echo "ro.build.otaurl=$UPDATEURL" >> $TEMP_FOLDER/system/build.prop
fi

#replace autostart app
if [ ! -z $AUTOSTART ]; then
echo "Changing autostart app..."
CURRENTAUTOSTART=$(grep ^ro.product.autostart $TEMP_FOLDER/system/build.prop)
if [ ! -z $CURRENTAUTOSTART ]; then
sed '/ro\.product\.autostart.*/d'  $TEMP_FOLDER/system/build.prop >  $TEMP_FOLDER/system/build.prop.new
rm -rf $TEMP_FOLDER/system/build.prop
mv $TEMP_FOLDER/system/build.prop.new $TEMP_FOLDER/system/build.prop
fi
echo "ro.product.autostart=$AUTOSTART" >> $TEMP_FOLDER/system/build.prop
fi

#replace product name
if [ ! -z $NAME ]; then
echo "Changing product name..."
CURRENTNAME=$(grep ^ro.product.name $TEMP_FOLDER/system/build.prop)
if [ ! -z $CURRENTNAME ]; then
sed '/ro\.product\.name.*/d'  $TEMP_FOLDER/system/build.prop >  $TEMP_FOLDER/system/build.prop.new
rm -rf $TEMP_FOLDER/system/build.prop
mv $TEMP_FOLDER/system/build.prop.new $TEMP_FOLDER/system/build.prop
fi
echo "ro.product.name=$NAME" >> $TEMP_FOLDER/system/build.prop
fi

if [ -z $DEVICE ]; then
echo "Getting ro.product.device..."
DEVICE=$(grep ^ro.product.device $TEMP_FOLDER/system/build.prop)
DEVICE=${DEVICE#*=}
fi
echo "OTA device name is \"$DEVICE\""

echo "Getting ro.build.date.utc..."
BUILD_VER=$(grep ^ro.build.date.utc $TEMP_FOLDER/system/build.prop)
BUILD_VER=${BUILD_VER#*=}
echo "ro.build.date.utc is \"$BUILD_VER\""

echo "Unmounting system_raw.img..."
umount $TEMP_FOLDER/system

echo "Compessing image into system.PARTITION..."
zerofree $TEMP_FOLDER/system_raw.img
$IMG2SIMG $TEMP_FOLDER/system_raw.img $TEMP_FOLDER/system.PARTITION
rm $TEMP_FOLDER/system_raw.img
rm -rf $TEMP_FOLDER/system

echo "Repacking image..."
$AML_IMAGE_PACKER -r $TEMP_FOLDER/image.cfg $TEMP_FOLDER $OUT_IMAGE

echo "Converting system.img into system.new.dat..."
mkdir -p $TEMP_FOLDER/ota/boot/ramdisk
mkdir -p $TEMP_FOLDER/ota/META-INF/com/google/android
$IMG2SDAT $TEMP_FOLDER/system.PARTITION $TEMP_FOLDER/ota/ 3
mv $TEMP_FOLDER/boot.PARTITION $TEMP_FOLDER/ota/boot.img
mv $TEMP_FOLDER/*.dtb $TEMP_FOLDER/ota/dtb.img
mv $TEMP_FOLDER/logo.PARTITION $TEMP_FOLDER/ota/logo.img
mv $TEMP_FOLDER/recovery.PARTITION $TEMP_FOLDER/ota/recovery.img

echo "Getting file_contexts from boot.img..."
$UNPACKBOOTIMG -i $TEMP_FOLDER/ota/boot.img -o $TEMP_FOLDER/ota/boot
CUR=$(pwd)
cd $TEMP_FOLDER/ota/boot/ramdisk
gunzip -c ../boot.img-ramdisk.gz | cpio -i
cd $CUR
mv $TEMP_FOLDER/ota/boot/ramdisk/file_contexts $TEMP_FOLDER/ota/file_contexts
rm -rf $TEMP_FOLDER/ota/boot

echo "Generating update binary and script..."
cp -f $UPDATER_BINARY $TEMP_FOLDER/ota/META-INF/com/google/android/update-binary
touch $TEMP_FOLDER/ota/META-INF/com/google/android/updater-script
echo "(!less_than_int($BUILD_VER, getprop(\"ro.build.date.utc\"))) || abort(\"Can't install this package $BUILD_VER over newer build (\" + getprop(\"ro.build.date.utc\") + \").\");" >> $TEMP_FOLDER/ota/META-INF/com/google/android/updater-script
echo "getprop(\"ro.product.device\") == \"$DEVICE\" || abort(\"This package is for \\\"$DEVICE\\\" devices; this is a \\\"\" + getprop(\"ro.product.device\") + \"\\\".\");" >> $TEMP_FOLDER/ota/META-INF/com/google/android/updater-script
echo "$(cat $UPDATER_SCRIPT)" >> $TEMP_FOLDER/ota/META-INF/com/google/android/updater-script
chmod 755 $TEMP_FOLDER/ota/META-INF/com/google/android/update-binary
chmod 755 $TEMP_FOLDER/ota/META-INF/com/google/android/updater-script

echo "Adding files into zip..."
cd $TEMP_FOLDER/ota
zip -r9 ota.zip ./
cd $CUR

echo "Signing generated zip..."
java -jar $SIGNAPK -w ${RECOVERY_KEY}.x509.pem ${RECOVERY_KEY}.pk8 $TEMP_FOLDER/ota/ota.zip $OTA

rm -rf $TEMP_FOLDER > /dev/null

chmod 666 $OTA
chmod 666 $OUT_IMAGE

echo "Done!"
